<?php

namespace Drupal\recurly_aegir\Processors;

use Drupal\recurly_aegir\Wrappers\SubscriptionWrapper;

/**
 * Processes Recurly notifications for Aegir.
 */
abstract class Processor {

  const NOTIFICATION_MAP = [
    'new_account_notification' => 'AccountNewProcessor',
    'canceled_account_notification' => 'AccountCanceledProcessor',
    'billing_info_updated_notification' => 'AccountUpdatedProcessor',
    'reactivated_account_notification' => 'AccountReactivatedProcessor',
    'new_invoice_notification' => 'InvoiceNewProcessor',
    'processing_invoice_notification' => 'InvoiceProcessingProcessor',
    'closed_invoice_notification' => 'InvoiceClosedProcessor',
    'past_due_invoice_notification' => 'InvoiceOverdueProcessor',
    'new_subscription_notification' => 'SubscriptionNewProcessor',
    'updated_subscription_notification' => 'SubscriptionUpdatedProcessor',
    'canceled_subscription_notification' => 'SubscriptionCanceledProcessor',
    'expired_subscription_notification' => 'SubscriptionExpiredProcessor',
    'renewed_subscription_notification' => 'SubscriptionRenewedProcessor',
    'scheduled_payment_notification' => 'PaymentScheduledProcessor',
    'processing_payment_notification' => 'PaymentProcessingProcessor',
    'successful_payment_notification' => 'PaymentSuccessfulProcessor',
    'failed_payment_notification' => 'PaymentFailedProcessor',
    'successful_refund_notification' => 'PaymentRefundSuccessfulProcessor',
    'void_payment_notification' => 'PaymentVoidProcessor',
  ];

  /**
   * Account information.
   *
   * @var SimpleXMLElement|null
   */
  protected $account;

  /**
   * Subscription information.
   *
   * @var Drupal\recurly_aegir\Wrappers\SubscriptionWrapper|null
   */
  protected $subscription;

  /**
   * Transaction information.
   *
   * @var SimpleXMLElement|null
   */
  protected $transaction;

  /**
   * Invoice information.
   *
   * @var SimpleXMLElement|null
   */
  protected $invoice;

  protected $result = FALSE;

  /**
   * Class constuctor.
   *
   * @param SimpleXMLElement|null $account
   *   Account information.
   * @param SimpleXMLElement|null $subscription
   *   Subscription information.
   * @param SimpleXMLElement|null $transaction
   *   Transaction information.
   * @param SimpleXMLElement|null $invoice
   *   Invoice information.
   */
  public function __construct(
      \SimpleXMLElement $account = NULL,
      \SimpleXMLElement $subscription = NULL,
      \SimpleXMLElement $transaction = NULL,
      \SimpleXMLElement $invoice = NULL) {
    $this->account = $account;
    $this->subscription = !empty($subscription->uuid) ? SubscriptionWrapper::get((string) $subscription->uuid) : NULL;
    $this->transaction = $transaction;
    $this->invoice = $invoice;
  }

  /**
   * Process any notifications coming from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function process();

  /**
   * Get the processing result.
   *
   * @return bool
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Returns the logging service object.
   */
  public static function getLogger() {
    return \Drupal::logger('recurly_aegir');
  }

  /**
   * Fetch the remote account code.
   *
   * @return string
   *   The remote account code.
   */
  protected function getAccountCode() {
    return (string) $this->account->account_code;
  }

  /**
   * Log a warning message and return the current object.
   */
  protected function failAndLogMessage($message) {
    $this->getLogger()->warning($message, [
      '%uuid' => $this->subscription->getUuid(),
      '%user' => $this->subscription->getUserId($this->getAccountCode(), TRUE),
    ]);

    $this->result = FALSE;
    return $this;
  }

}
