<?php

namespace Drupal\recurly_aegir\Provisioners;

/**
 * Base class for provisioning things via Aegir's Web service API.
 */
abstract class Provisioner {

  /**
   * The response data received after sending a provision request.
   *
   * @var array
   */
  protected $response;

  /**
   * The response headers received after sending a provision request.
   *
   * @var array
   */
  protected $headers;

  /**
   * The response code received after sending a provision request.
   *
   * @var int
   */
  protected $status;

  /**
   * Sends a request to Aegir to provision something.
   *
   * Implementing classes must do the following:
   *   1. Make a service call to Aegir that creates a task to get/do something.
   *   2. Set $this->$task_id to the task ID returned in the call (if possible).
   *   3. Return $this for method chaining.
   *
   * @return $this
   *   The object itself.
   */
  abstract protected function provision();

  /**
   * Returns the endpoint for the Aegir Web service.
   *
   * @return string
   *   The endpoint URL.
   */
  public static function getAegirServiceEndpoint() {
    $endpoint = \Drupal::config('recurly_aegir.settings')->get('service_endpoint_url');
    if (empty($endpoint)) {
      $this->getLogger()->critical('The Aegir service endpoint has not been configured.');
    }
    return $endpoint;
  }

  /**
   * Sends a request to the Aegir host, and sets class variables with results.
   *
   * @param string $resource
   *   The endpoint resource to query.
   * @param array $data
   *   The data to send.
   */
  protected function sendRequestAndReceiveResponse($resource, array $data) {
    $target = $this->getAegirServiceEndpoint() . '/' . $resource;

    $this->logRequest($resource, $data);

    try {
      $request = empty($data) ? $this->sendGetRequest($target) : $this->sendPostRequest($target, $data);
    }
    catch (Exception $e) {
      watchdog_exception('recurly_aegir', $e->getMessage());
    }

    $this->status = $request->getStatusCode();
    $this->headers = $request->getHeaders();
    $this->response = json_decode((string) $request->getBody(), TRUE);
  }

  /**
   * Log the request before sending it.
   */
  protected function logRequest($resource, $data) {
    $this->getLogger()
      ->info('Sending remote request for resource "%resource" with headers "%headers", arguments "%arguments" and data "%data"', [
        '%resource' => $resource,
        '%headers' => serialize($this->getHeadersToSend(TRUE)),
        '%arguments' => serialize($this->getQueryParametersToSend()),
        '%data' => serialize($data),
      ]);
  }

  /**
   * Send a GET request.
   */
  protected function sendGetRequest($target) {
    return $this->getWebServiceClient()->get($target, [
      'headers' => $this->getHeadersToSend(),
      'query' => $this->getQueryParametersToSend(),
    ]);
  }

  /**
   * Send a POST request.
   */
  protected function sendPostRequest($target, $data) {
    return $this->getWebServiceClient()->post($target, [
      'headers' => $this->getHeadersToSend(),
      'json' => $data,
    ]);
  }

  /**
   * Fetches a useable Web service client.
   */
  protected function getWebServiceClient() {
    return \Drupal::httpClient();
  }

  /**
   * Fetches the headers to send on a request.
   */
  protected function getHeadersToSend($skip_key = FALSE) {
    return $skip_key ? [] : [
      'API-KEY' => \Drupal::config('recurly_aegir.settings')->get('service_endpoint_key'),
    ];
  }

  /**
   * Fetches the query parameters to send in the request.
   */
  protected function getQueryParametersToSend() {
    return [];
  }

  /**
   * Fetches the request response.
   */
  public function getResponse() {
    return $this->response;
  }

  /**
   * Get the action performed by this provisioner.
   */
  public static function getActionPerformed() {
    return static::ACTION_PERFORMED;
  }

  /**
   * Record a message about the successful action in the log.
   *
   * @return $this
   *   The object itself.
   */
  abstract protected function recordSuccessLogMessage();

  /**
   * Fetches the name of the current class.
   */
  protected function getClassName() {
    return (new \ReflectionClass($this))->getShortName();
  }

  /**
   * Perform an action using the remote Aegir service and log the results.
   *
   * As we may be in the middle of a save operation, or want to run several
   * tasks without saving the log every time, we should not save the site after
   * we update its task log here. This must be done by the caller.
   *
   * @return $this
   *   The object itself.
   */
  public function performActionAndLogResults() {
    try {
      $this->provision()->recordSuccessLogMessage();
    }
    catch (Exception $e) {
      watchdog_exception('recurly_aegir', $e);
    }
    return $this;
  }

  /**
   * Returns the logging service object.
   */
  public static function getLogger() {
    return \Drupal::logger('recurly_aegir');
  }

}
